<?xml version="1.0"?>

<st-source>
<time-stamp>From VisualWorks® NonCommercial, Release 5i.4 of August 9, 2001 on May 27, 2002 at 12:40:11 am</time-stamp>


<class>
<name>Sender</name>
<environment>Smalltalk</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>chat </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>Valeko</category>
</class>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>Sender class</class-id> <category>instance creation</category>

<body>new: aChat
	| newSender |
	newSender := self new.
	newSender parent: aChat.
	^newSender.</body>
</methods>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>Sender</class-id> <category>communication</category>

<body>connect: aHost port: aPort
	"connect to network via the host and port"

	| stream item |
	[ stream := self request: 'GET' host: aHost port: aPort.]
	on: OsError do: [ :ex|
		Dialog warn: 'Connection failed'.
		^nil.
	].
	[stream atEnd] whileFalse: [
		stream upTo: Character space.
		item := HostItem readFrom: stream.
		stream upTo: Character cr.
		chat hostList add: item.
	].

	self requestOthers: 'CONNECT ',(chat hostList me asString).</body>

<body>disconnect
	"send a DISCONNECT message to other chat users"

	self requestOthers: 'DISCONNECT ',(chat hostList me nick).</body>

<body>message: text
	"send a message to other chat users"

	chat hostList me isNil ifTrue:[^nil].
	text = '' ifTrue:[^nil].
	self requestAll: 'MESSAGE ',(chat hostList me asString),' ',text.</body>

<body>request: request host: host port: port
	"send a request to one host and return the result"

	| sockAccessor stream line result |
	sockAccessor := SocketAccessor newTCPclientToHost: host port: port.
	stream := sockAccessor readAppendStream.
	stream lineEndCRLF.
	stream nextPutAll: request; cr; commit.
	result := WriteStream on: String new.
	[stream atEnd] whileFalse: [
		line := stream upTo: Character cr.
		(line = 'OK') ifFalse: [ result nextPutAll: line; cr; commit].
	].
	stream close.
	^result contents readStream.</body>

<body>request: request multiple: list 
	"sends a request to multiple chat users given a list of them"

	list do: [:item |
		[self request: request host: item host port: item port]
		on: OsError
		do: 	[:ex | 
			chat hostList removeNick: item nick.
			chat writeLine: item nick , ' IS UNREACHABLE'
			]
		]</body>

<body>requestAll: request 
	"send a request to all chat users"

	self request: request multiple: chat hostList valuesAll.</body>

<body>requestOthers: request 
	"send a request to all chat users"

	self request: request multiple: chat hostList valuesOthers</body>
</methods>

<methods>
<class-id>Sender</class-id> <category>initialization</category>

<body>parent: parent
	chat := parent</body>
</methods>


<class>
<name>Receiver</name>
<environment>Smalltalk</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>chat server </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>Valeko</category>
</class>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>Receiver class</class-id> <category>instance creation</category>

<body>new: aPort parent: aChat 
	"create a new instance of receiver, listening at given port"

	| newReceiver |
	newReceiver := self new.
	(newReceiver startServer: aPort parent: aChat) isNil
		ifTrue: [^nil]
		ifFalse: [^newReceiver]</body>
</methods>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>Receiver</class-id> <category>messages</category>

<body>CONNECT: aLine 
	| stream |
	stream := aLine readStream.
	(stream upTo: Character space) = 'CONNECT' ifFalse: [^self].
	chat hostList add: (HostItem readFrom: stream)</body>

<body>DISCONNECT: aLine
	| stream |
	stream := aLine readStream.
	((stream upTo: Character space) = 'DISCONNECT') ifFalse: [^self].
	chat hostList removeNick: (stream upTo: Character space).</body>

<body>GET: aLine result: result 
	| stream |
	stream := aLine readStream.
	(stream upTo: Character space) = 'GET' ifFalse: [^self].
	chat hostList printOn: result</body>

<body>MESSAGE: aLine
	| text stream item |
	stream := aLine readStream.
	((stream upTo: Character space) = 'MESSAGE') ifFalse: [^self].
	item := HostItem readFrom: stream.
	text := (stream upToEnd).
	chat hostList add: item.
	chat writeLine: item nick,' :',text.</body>

<body>processMessage: stream 
	"process an incoming message"

	| request |
	request := stream upTo: Character cr.
	self GET: request result: stream.
	self CONNECT: request.
	self DISCONNECT: request.
	self MESSAGE: request</body>
</methods>

<methods>
<class-id>Receiver</class-id> <category>communication</category>

<body>serverLoop
	"server loop, which accepts connections"

	| stream |
	[ | acceptedSocket |
		acceptedSocket := server accept.
		stream := acceptedSocket readAppendStream.
		stream lineEndCRLF.
		self processMessage: stream.
		stream nextPutAll: 'OK'; cr ; commit.
		stream close.
	] repeat.</body>

<body>startServer: aPort parent: aChat 
	"start the server and fork out a thread for accepting connections"

	chat := aChat.
	
	[server := SocketAccessor newTCPserverAtPort: aPort.
	server listenFor: 5]
		on: OsError
		do: 
			[:ex | 
			Dialog warn: 'Error opening port for listening'.
			^nil].
	[self serverLoop]
		forkAt: Processor userSchedulingPriority - 1.
	^self</body>

<body>stopServer
	"stop the server by closing the listening socket"

	server isNil ifFalse: [server close]</body>
</methods>


<class>
<name>HostList</name>
<environment>Smalltalk</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>me dictionary </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>Valeko</category>
</class>

<methods>
<class-id>HostList</class-id> <category>initialization</category>

<body>list
	^dictionary isNil
		ifTrue:
			[dictionary := Dictionary new]
		ifFalse:
			[dictionary]</body>
</methods>

<methods>
<class-id>HostList</class-id> <category>information</category>

<body>me
	^me</body>

<body>nicks
	"return keys (nicks) with my nick included"

	| nicks |
	nicks := self list keys asSortedCollection.
	self me isNil ifFalse: [nicks add: self me nick].
	^nicks.</body>

<body>printOn: stream 
	"dump the host list in a text form into a stream"

	me printOn: stream.
	self list do: [:value | value printOn: stream]</body>

<body>valuesAll
	"return values (list items) with my nick included at the end"

	| values |
	values := self list asOrderedCollection.
	self me isNil ifFalse: [values addLast: self me].
	^values.</body>

<body>valuesOthers
	"return values (list items) with my nick NOT included at the end"

	| values |
	values := self list asOrderedCollection.
	^values.</body>
</methods>

<methods>
<class-id>HostList</class-id> <category>operations</category>

<body>add: item 
	"add a new item to the list"

	(item nick) = self me nick ifTrue: [^nil].
	self list at: (item nick) put: item.
	self changed: #hostList</body>

<body>addNick: aNick host: aHost port: aPort 
	"add a new item to the list"

	| item |
	item := HostItem nick: aNick host: aHost port: aPort.
	self add: item</body>

<body>removeAll
	"remove all data from the list"

	dictionary := nil.
	self changed: #hostList</body>

<body>removeMe
	"clear my identity"

	me := nil.
	self changed: #hostList</body>

<body>removeNick: aNick 
	"remove an item by nick name"

	self list removeKey: aNick ifAbsent: [].
	self changed: #hostList</body>

<body>setMe: aNick host: aHost port: aPort 
	"set my identity"

	me := HostItem nick: aNick host: aHost port: aPort.
	self changed: #hostList</body>
</methods>


<class>
<name>HostItem</name>
<environment>Smalltalk</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>nick host port </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>Valeko</category>
</class>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>HostItem class</class-id> <category>instance creation</category>

<body>nick: aNick host: aHost port: aPort 
	"return a new object with fields filled in with parameters"

	| item |
	item := HostItem new.
	item nick: aNick host: aHost port: aPort.
	^item</body>

<body>readFrom: stream
	"read object's variables from a character stream"

	^self nick: (stream upTo: Character space) host: (stream upTo: Character space)
	port: (Integer readFrom: stream).</body>
</methods>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>HostItem</class-id> <category>initialization</category>

<body>nick: aNick host: aHost port: aPort 
	"set object's variables to parameter values"

	nick := aNick.
	host := aHost.
	port := aPort</body>

<body>readFrom: stream
	"read object's variables from a character stream"

	nick := (stream upTo: Character space).
	host := (stream upTo: Character space).
	port := (Integer readFrom: stream).</body>
</methods>

<methods>
<class-id>HostItem</class-id> <category>information</category>

<body>asString
	"return this item as a string"

	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: nick , ' ' , host , ' ' , self portAsString.
	^stream contents</body>

<body>host
	^host.</body>

<body>hostAsName
	^IPSocketAddress hostNameByAddress: (IPSocketAddress hostAddressByName: host)</body>

<body>nick
	^nick.</body>

<body>port
	^port.</body>

<body>portAsString
	"return the port number as a string"

	| stream |
	stream := WriteStream on: String new.
	port printOn: stream.
	^stream contents</body>

<body>printOn: stream 
	"dump item's contents as a line to a stream"

	stream nextPutAll: 'ITEM ',self asString; cr; commit</body>
</methods>


<class>
<name>Chat</name>
<environment>Smalltalk</environment>
<super>UI.ApplicationModel</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>receiver sender hostList interfaces textArea inputField userList </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>Valeko</category>
</class>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>Chat class</class-id> <category>interface specs</category>

<body>windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	&lt;resource: #canvas&gt;
	^#(#{UI.FullSpec} 
		#window: 
		#(#{UI.WindowSpec} 
			#label: 'Chat' 
			#bounds: #(#{Graphics.Rectangle} 338 300 800 528 ) ) 
		#component: 
		#(#{UI.SpecCollection} 
			#collection: #(
				#(#{UI.InputFieldSpec} 
					#layout: #(#{Graphics.LayoutFrame} 10 0 -35 1 -170 1 -10 1 ) 
					#name: #InputField 
					#model: #inputField ) 
				#(#{UI.ActionButtonSpec} 
					#layout: #(#{Graphics.LayoutFrame} -160 1 -35 1 -120 1 -10 1 ) 
					#name: #SendButton 
					#model: #send 
					#label: 'Send' 
					#isDefault: true 
					#defaultable: true ) 
				#(#{UI.SequenceViewSpec} 
					#layout: #(#{Graphics.LayoutFrame} -110 1 10 0 -10 1 -135 1 ) 
					#name: #UserList 
					#model: #userList 
					#useModifierKeys: true 
					#selectionType: #highlight ) 
				#(#{UI.ActionButtonSpec} 
					#layout: #(#{Graphics.LayoutFrame} -100 1 -125 1 -20 1 -100 1 ) 
					#name: #SetupButton 
					#model: #setup 
					#label: 'Setup' 
					#isDefault: false 
					#defaultable: true ) 
				#(#{UI.ActionButtonSpec} 
					#layout: #(#{Graphics.LayoutFrame} -100 1 -95 1 -20 1 -70 1 ) 
					#name: #ConnectButton 
					#model: #connect 
					#label: 'Connect' 
					#defaultable: true ) 
				#(#{UI.ActionButtonSpec} 
					#layout: #(#{Graphics.LayoutFrame} -100 1 -65 1 -20 1 -40 1 ) 
					#name: #DisconnectButton 
					#model: #disconnect 
					#label: 'Disconnect' 
					#defaultable: true ) 
				#(#{UI.ActionButtonSpec} 
					#layout: #(#{Graphics.LayoutFrame} -100 1 -35 1 -20 1 -10 1 ) 
					#name: #CloseButton 
					#model: #close 
					#label: 'Close' 
					#defaultable: true ) 
				#(#{UI.TextEditorSpec} 
					#layout: #(#{Graphics.LayoutFrame} 10 0 10 0 -120 1 -45 1 ) 
					#name: #TextArea 
					#flags: 15 
					#model: #textArea 
					#isReadOnly: true 
					#tabRequiresControl: true 
					#isWrapped: false ) ) ) )</body>
</methods>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>Chat</class-id> <category>aspects</category>

<body>inputField
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^inputField isNil
		ifTrue:
			[inputField := String new asValue]
		ifFalse:
			[inputField]</body>

<body>textArea
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^textArea isNil
		ifTrue:
			[textArea := String new asValue]
		ifFalse:
			[textArea]</body>

<body>userList
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^userList isNil
		ifTrue:
			[userList := SelectionInList new]
		ifFalse:
			[userList]</body>
</methods>

<methods>
<class-id>Chat</class-id> <category>network</category>

<body>localHostAddress
	"return a local host IP address as a string"

	| ip stream |
	ip := (IPSocketAddress hostAddressByName: SocketAccessor getHostname).
	stream := WriteStream on: (String new).
	(1 to: 3) do: [:i | (ip at: i) printOn: stream. stream nextPut: $.].
	(ip at: 4) printOn: stream.
	^stream contents.</body>
</methods>

<methods>
<class-id>Chat</class-id> <category>actions</category>

<body>close
	"disconnect and close local server"

	self disconnect.
	receiver isNil ifFalse: [
		receiver stopServer.
		receiver := nil.
	].
	self hostList removeMe.
	self clearTextArea.
	self setTitle: 'Chat'.</body>

<body>connect
	"ask for a host and a port and connect to network"

	| host result port |
	receiver isNil
		ifTrue: [self setup isNil ifTrue: [^nil]]
		ifFalse: [self disconnect].
	result := self connectDialog.
	result isNil ifTrue: [^nil].
	host := result at: 'host'.
	port := result at: 'port'.
	self sender connect: host port: port</body>

<body>disconnect
	"disconnect from network"

	receiver isNil ifTrue: [^nil].
	self sender disconnect.
	self hostList removeAll.
	self clearTextArea.</body>

<body>send
	"send out a message"

	self sender message: self inputField value.
	self inputField value: ''.</body>

<body>setup
	"asks for a nickname and local port and start listening for incomning connection"

	| result nick port me |
	self close.
	result := self setupDialog.
	result isNil ifTrue: [^nil].
	nick := result at: 'nick'.
	port := result at: 'port'.
	receiver := Receiver new: port parent: self.
	receiver isNil ifFalse: [
		self hostList	setMe: nick	host: self localHostAddress port: port.
		me := self hostList me.
		self setTitle: (me nick),' at ',(me hostAsName),':',(me portAsString).
	]</body>
</methods>

<methods>
<class-id>Chat</class-id> <category>overriden</category>

<body>changeRequest
	"closes server when the application window is closed"

	self close.
	^true.</body>

<body>openInterface: aSymbol withPolicy: aPolicy 
	"overrides the method of UI.ApplicationModel"

	| bldr |
	bldr := super openInterface: aSymbol withPolicy: aPolicy.
	self interfaces add: bldr.
	^bldr</body>

<body>update: symbol 
	"receives a change notification and updates the specified control"

	symbol = #hostList ifTrue: [userList list: hostList nicks]</body>
</methods>

<methods>
<class-id>Chat</class-id> <category>interface</category>

<body>clearTextArea
	"clears the textArea"

	self textArea value: ''</body>

<body>connectDialog
	"ask user for parameters required to connect to network"

	| port result host |
	host := Dialog
				request: 'Connect to host'
				initialAnswer: ''
				onCancel: ['']
				windowLabel: 'Connect'
				for: Dialog defaultParentWindow.
	host = '' ifTrue: [^nil].
	port := Dialog
				request: 'Connect to port'
				initialAnswer: '6060'
				onCancel: ['']
				windowLabel: 'Connect'
				for: Dialog defaultParentWindow.
	port = '' ifTrue: [^nil].
	port := Integer readFrom: port readStream.
	result := Dictionary new.
	result at: 'host' put: host; at: 'port' put: port.
	^result.</body>

<body>setTitle: title 
	"set a title of all interfaces"

	self interfaces do: [:bldr | bldr window label: title]</body>

<body>setupDialog
	"ask user for parameters required to set up a local server"

	| nick port result |
	nick := Dialog
				request: 'Your nickname'
				initialAnswer: ''
				onCancel: ['']
				windowLabel: 'Setup'
				for: Dialog defaultParentWindow.
	nick = '' ifTrue: [^nil].
	port := Dialog
				request: 'Your port'
				initialAnswer: '6060'
				onCancel: ['']
				windowLabel: 'Setup'
				for: Dialog defaultParentWindow.
	port = '' ifTrue: [^nil].
	port := Integer readFrom: port readStream.
	result := Dictionary new.
	result at: 'nick' put: nick; at: 'port' put: port.
	^result.</body>

<body>writeLine: aLine
	"writes a line to textArea and scroll it down"

"	self textArea value:(self textArea value,aLine,(String with: Character cr))."
"	self scrollTextArea."

	| editor |
	self interfaces do: [:bldr |
		editor := (bldr componentAt: #TextArea) widget controller.
		editor selectAt: (editor text size + 1).
		editor replaceSelectionWith: aLine,(String with: Character cr).
		editor deselect.
	]</body>
</methods>

<methods>
<class-id>Chat</class-id> <category>initialization</category>

<body>hostList
	hostList isNil
		ifTrue: 
			[hostList := HostList new.
			hostList addDependent: self].
	^hostList</body>

<body>interfaces
	interfaces isNil ifTrue: [interfaces := Set new].
	^interfaces</body>

<body>sender
	sender isNil ifTrue: [sender := Sender new: self].
	^sender</body>
</methods>



</st-source>
